"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ServerQuery = void 0;
exports.createRequest = createRequest;
var _assert = _interopRequireDefault(require("assert"));
var _debug = _interopRequireDefault(require("debug"));
var _got = _interopRequireDefault(require("got"));
var _lodash = require("lodash");
var _core = require("@verdaccio/core");
var _testHelper = require("@verdaccio/test-helper");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const debug = (0, _debug.default)('verdaccio:registry:request');
class RequestAssert {
  constructor(response) {
    this.response = response;
  }
  status(code) {
    debug('expected check status %s vs response code %s', code, this.response.statusCode);
    (0, _assert.default)(code === this.response.statusCode);
    return this;
  }
  equal_body(expected) {
    _assert.default.strictEqual(expected, this.response.body);
  }
  body_ok(expected) {
    debug('body expect ok %s', expected);
    if ((0, _lodash.isRegExp)(expected)) {
      var _this$response$body, _this$response$body$o, _this$response$body2;
      (0, _assert.default)((_this$response$body = this.response.body) === null || _this$response$body === void 0 ? void 0 : (_this$response$body$o = _this$response$body.ok) === null || _this$response$body$o === void 0 ? void 0 : _this$response$body$o.match(expected));
      (0, _assert.default)((_this$response$body2 = this.response.body) === null || _this$response$body2 === void 0 ? void 0 : _this$response$body2.ok.match(expected), `'${this.response.body.ok}' doesn't match " ${expected}`);
    } else if (typeof expected === 'string') {
      var _this$response$body3;
      _assert.default.equal((_this$response$body3 = this.response.body) === null || _this$response$body3 === void 0 ? void 0 : _this$response$body3.ok, expected);
    } else {
      _assert.default.deepEqual(this.response.body, expected);
    }
  }
  body_error(expected) {
    var _this$response$body6;
    debug('body expect error %s', expected);
    if ((0, _lodash.isRegExp)(expect)) {
      var _this$response, _this$response$body4, _this$response$body4$, _this$response$body5;
      (0, _assert.default)((_this$response = this.response) === null || _this$response === void 0 ? void 0 : (_this$response$body4 = _this$response.body) === null || _this$response$body4 === void 0 ? void 0 : (_this$response$body4$ = _this$response$body4.error) === null || _this$response$body4$ === void 0 ? void 0 : _this$response$body4$.match(expected), `${(_this$response$body5 = this.response.body) === null || _this$response$body5 === void 0 ? void 0 : _this$response$body5.error} doesn't match ${expected}`);
    }
    _assert.default.equal((_this$response$body6 = this.response.body) === null || _this$response$body6 === void 0 ? void 0 : _this$response$body6.ok, null);
  }
}
async function createRequest(options) {
  var _options$method;
  debug('options %s', JSON.stringify(options));
  let body = undefined;
  if ((0, _lodash.isNil)(options.body) === false) {
    // @ts-ignore
    body = (0, _lodash.isObject)(options.body) === false ? JSON.stringify(options.body) : options.body;
  }
  const method = options === null || options === void 0 ? void 0 : (_options$method = options.method) === null || _options$method === void 0 ? void 0 : _options$method.toLocaleLowerCase();
  debug('method %s', method);
  debug('url %s', options === null || options === void 0 ? void 0 : options.url);
  debug('headers %s', options === null || options === void 0 ? void 0 : options.headers);
  if (method === 'get') {
    var _options$encoding;
    return (0, _got.default)(options.url, {
      isStream: false,
      resolveBodyOnly: false,
      throwHttpErrors: false,
      // @ts-ignore
      responseType: (_options$encoding = options.encoding) !== null && _options$encoding !== void 0 ? _options$encoding : 'json',
      headers: options.headers,
      method: options.method,
      body,
      retry: {
        limit: 0
      }
      // @ts-ignore
    }).then(response => {
      return new RequestAssert(response);
    });
  } else if (method === 'put') {
    return _got.default.put(options.url, {
      throwHttpErrors: false,
      responseType: 'json',
      headers: options.headers,
      json: options.body ? options.body : undefined,
      retry: {
        limit: 0
      }
    })
    // @ts-ignore
    .then(response => {
      return new RequestAssert(response);
    });
  } else if (method === 'delete') {
    return _got.default.delete(options.url, {
      throwHttpErrors: false,
      responseType: 'json',
      headers: options.headers,
      retry: {
        limit: 0
      }
    })
    // @ts-ignore
    .then(response => {
      return new RequestAssert(response);
    });
  }
}
class ServerQuery {
  constructor(url) {
    this.url = url.replace(/\/$/, '');
    debug('server url %s', this.url);
    this.userAgent = 'node/v14.1.2 linux x64';
  }
  request(options) {
    return createRequest({
      ...options,
      url: `${this.url}${options.uri}`
    });
  }
  debug() {
    return this.request({
      uri: '/-/_debug',
      method: 'get',
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON
      }
    });
  }

  /**
   *
   *
   * @param {{ name: string; password: string }} { name, password }
   * @return {*}  {Promise<ResponseAssert>}
   * @memberof ServerQuery
   * @deprecated use createUser instead
   */
  auth({
    name,
    password
  }) {
    return this.createUser(name, password);
  }
  createUser(name, password) {
    return this.request({
      uri: `/-/user/org.couchdb.user:${encodeURIComponent(name)}`,
      method: 'PUT',
      body: {
        name,
        password,
        _id: `org.couchdb.user:${name}`,
        type: 'user',
        roles: [],
        date: new Date()
      }
    });
  }
  logout(token) {
    return this.request({
      uri: `/-/user/token/${encodeURIComponent(token)}`,
      method: 'DELETE'
    });
  }
  getPackage(name) {
    return this.request({
      uri: `/${encodeURIComponent(name)}`,
      method: 'get'
    });
  }
  getTarball(name, filename) {
    return this.request({
      uri: `/${encodeURIComponent(name)}/-/${encodeURIComponent(filename)}`,
      method: 'GET',
      encoding: 'buffer'
    });
  }

  /**
   * Remove entire package.
   * @param name package name
   * @param rev revision id
   * @returns
   */
  removePackage(name, rev) {
    return this.request({
      uri: `/${encodeURIComponent(name)}/-rev/${rev}`,
      method: 'DELETE',
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON_CHARSET
      }
    });
  }
  removeSingleTarball(name, filename) {
    return this.request({
      uri: `/${encodeURIComponent(name)}/-/${filename}/-rev/whatever`,
      method: 'DELETE',
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON_CHARSET
      }
    });
  }

  /**
   *
   * @param name
   * @param tag
   * @param version
   * @returns
   */
  addTag(name, tag, version) {
    return this.request({
      uri: `/${encodeURIComponent(name)}/${encodeURIComponent(tag)}`,
      method: 'PUT',
      body: version,
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON
      }
    });
  }
  putVersion(name, version, data, headers) {
    return this.request({
      uri: `/${encodeURIComponent(name)}/${encodeURIComponent(version)}/-tag/latest`,
      method: 'PUT',
      body: data,
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON,
        ...headers
      }
    });
  }
  putPackage(name, data, headers = {}) {
    return this.request({
      uri: `/${encodeURIComponent(name)}`,
      method: 'PUT',
      body: data,
      headers: {
        [_core.HEADERS.CONTENT_TYPE]: _core.HEADERS.JSON,
        ...headers
      }
    });
  }
  async addPackage(name, version = '1.0.0', message = _core.API_MESSAGE.PKG_CREATED) {
    return (await this.putPackage(name, (0, _testHelper.generatePackageMetadata)(name, version))).status(_core.HTTP_STATUS.CREATED).body_ok(message);
  }
  async addPackageAssert(name, version = '1.0.0') {
    return this.putPackage(name, (0, _testHelper.generatePackageMetadata)(name, version));
  }
  async whoami() {
    debug('request whoami');
    return await this.request({
      uri: '/-/whoami',
      method: 'get'
    });
  }
  async ping() {
    return (await this.request({
      uri: '/-/ping',
      method: 'get'
    })).status(_core.HTTP_STATUS.OK);
  }
}
exports.ServerQuery = ServerQuery;
//# sourceMappingURL=request.js.map